/**
 * Copyright (c) 2014, 2015 Christian W. Damus and others.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   Christian W. Damus - Initial API and implementation
 */
package org.eclipse.papyrus.uml.profile.types.generator;

import com.google.common.base.Objects;
import com.google.common.collect.Iterables;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.inject.Inject;
import javax.inject.Singleton;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.gmf.runtime.emf.type.core.ElementTypeRegistry;
import org.eclipse.gmf.runtime.emf.type.core.IElementType;
import org.eclipse.gmf.runtime.emf.type.core.IMetamodelType;
import org.eclipse.gmf.runtime.emf.type.core.ISpecializationType;
import org.eclipse.papyrus.infra.types.ElementTypeConfiguration;
import org.eclipse.papyrus.infra.types.ElementTypeSetConfiguration;
import org.eclipse.papyrus.infra.types.ElementTypesConfigurationsFactory;
import org.eclipse.papyrus.infra.types.IconEntry;
import org.eclipse.papyrus.infra.types.MetamodelTypeConfiguration;
import org.eclipse.papyrus.infra.types.SpecializationTypeConfiguration;
import org.eclipse.papyrus.infra.types.core.registries.ElementTypeSetConfigurationRegistry;
import org.eclipse.papyrus.uml.profile.types.generator.Identifiers;
import org.eclipse.papyrus.uml.profile.types.generator.UML;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.xtext.xbase.lib.Extension;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.ListExtensions;
import org.eclipse.xtext.xbase.lib.ObjectExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;
import org.eclipse.xtext.xbase.lib.StringExtensions;

/**
 * Utility extensions for working with and generating objects for the base UML element types specialized by the profile.
 */
@Singleton
@SuppressWarnings("all")
public class UMLElementTypes {
  private final static Pattern VISUAL_ID_PATTERN = Pattern.compile("\\d{4}");
  
  @Extension
  private static ElementTypesConfigurationsFactory elementtypesconfigurationsFactory = ElementTypesConfigurationsFactory.eINSTANCE;
  
  @Inject
  @Extension
  private Identifiers _identifiers;
  
  @Inject
  @Extension
  private UML _uML;
  
  public String getElementTypeID(final org.eclipse.uml2.uml.Class metaclass) {
    String _name = metaclass.getName();
    return ("org.eclipse.papyrus.uml." + _name);
  }
  
  public ElementTypeConfiguration getElementTypeConfiguration(final org.eclipse.uml2.uml.Class metaclass) {
    ElementTypeSetConfiguration _baseUMLElementTypeSet = this.getBaseUMLElementTypeSet();
    EList<ElementTypeConfiguration> _elementTypeConfigurations = _baseUMLElementTypeSet.getElementTypeConfigurations();
    final Function1<ElementTypeConfiguration, Boolean> _function = new Function1<ElementTypeConfiguration, Boolean>() {
      public Boolean apply(final ElementTypeConfiguration it) {
        String _identifier = it.getIdentifier();
        String _elementTypeID = UMLElementTypes.this.getElementTypeID(metaclass);
        return Boolean.valueOf(Objects.equal(_identifier, _elementTypeID));
      }
    };
    return IterableExtensions.<ElementTypeConfiguration>findFirst(_elementTypeConfigurations, _function);
  }
  
  public ElementTypeSetConfiguration getBaseUMLElementTypeSet() {
    ElementTypeSetConfigurationRegistry _instance = ElementTypeSetConfigurationRegistry.getInstance();
    Map<String, Map<String, ElementTypeSetConfiguration>> _elementTypeSetConfigurations = _instance.getElementTypeSetConfigurations();
    String _contextId = this._identifiers.getContextId();
    Map<String, ElementTypeSetConfiguration> _get = _elementTypeSetConfigurations.get(_contextId);
    String _umlElementTypesSet = this._identifiers.getUmlElementTypesSet();
    return _get.get(_umlElementTypesSet);
  }
  
  public Iterable<IElementType> getBaseUMLElementTypes() {
    ElementTypeSetConfiguration _baseUMLElementTypeSet = this.getBaseUMLElementTypeSet();
    EList<ElementTypeConfiguration> _elementTypeConfigurations = _baseUMLElementTypeSet.getElementTypeConfigurations();
    final Function1<ElementTypeConfiguration, IElementType> _function = new Function1<ElementTypeConfiguration, IElementType>() {
      public IElementType apply(final ElementTypeConfiguration it) {
        ElementTypeRegistry _instance = ElementTypeRegistry.getInstance();
        String _identifier = it.getIdentifier();
        return _instance.getType(_identifier);
      }
    };
    List<IElementType> _map = ListExtensions.<ElementTypeConfiguration, IElementType>map(_elementTypeConfigurations, _function);
    return IterableExtensions.<IElementType>filterNull(_map);
  }
  
  public EClass getEClass(final org.eclipse.uml2.uml.Class metaclass) {
    String _name = metaclass.getName();
    EClassifier _eClassifier = UMLPackage.eINSTANCE.getEClassifier(_name);
    return ((EClass) _eClassifier);
  }
  
  public IconEntry getIconEntry(final org.eclipse.uml2.uml.Class metaclass) {
    ElementTypeConfiguration _elementTypeConfiguration = this.getElementTypeConfiguration(metaclass);
    IconEntry _iconEntry = null;
    if (_elementTypeConfiguration!=null) {
      _iconEntry=_elementTypeConfiguration.getIconEntry();
    }
    return this.copy(_iconEntry);
  }
  
  private IconEntry copy(final IconEntry prototype) {
    IconEntry _createIconEntry = UMLElementTypes.elementtypesconfigurationsFactory.createIconEntry();
    final Procedure1<IconEntry> _function = new Procedure1<IconEntry>() {
      public void apply(final IconEntry it) {
        String _bundleId = prototype.getBundleId();
        it.setBundleId(_bundleId);
        String _iconPath = prototype.getIconPath();
        it.setIconPath(_iconPath);
      }
    };
    return ObjectExtensions.<IconEntry>operator_doubleArrow(_createIconEntry, _function);
  }
  
  public ElementTypeSetConfiguration getBaseElementTypeSet() {
    ElementTypeSetConfiguration _elvis = null;
    ElementTypeSetConfiguration _baseElementTypesSetConfiguration = this._identifiers.getBaseElementTypesSetConfiguration();
    if (_baseElementTypesSetConfiguration != null) {
      _elvis = _baseElementTypesSetConfiguration;
    } else {
      ElementTypeSetConfigurationRegistry _instance = ElementTypeSetConfigurationRegistry.getInstance();
      Map<String, Map<String, ElementTypeSetConfiguration>> _elementTypeSetConfigurations = _instance.getElementTypeSetConfigurations();
      String _contextId = this._identifiers.getContextId();
      Map<String, ElementTypeSetConfiguration> _get = _elementTypeSetConfigurations.get(_contextId);
      String _baseElementTypesSet = this._identifiers.getBaseElementTypesSet();
      ElementTypeSetConfiguration _get_1 = _get.get(_baseElementTypesSet);
      _elvis = _get_1;
    }
    return _elvis;
  }
  
  public Iterable<ElementTypeConfiguration> getBaseElementTypes() {
    ElementTypeSetConfiguration _baseElementTypeSet = this.getBaseElementTypeSet();
    EList<ElementTypeConfiguration> _elementTypeConfigurations = _baseElementTypeSet.getElementTypeConfigurations();
    final Function1<ElementTypeConfiguration, Boolean> _function = new Function1<ElementTypeConfiguration, Boolean>() {
      public Boolean apply(final ElementTypeConfiguration it) {
        return Boolean.valueOf(UMLElementTypes.this.validType(it));
      }
    };
    return IterableExtensions.<ElementTypeConfiguration>filter(_elementTypeConfigurations, _function);
  }
  
  public boolean validType(final ElementTypeConfiguration elementType) {
    EClass _metaclass = this.getMetaclass(elementType);
    return (!Objects.equal(_metaclass, null));
  }
  
  private boolean isDiagramSpecific() {
    ElementTypeSetConfiguration _baseElementTypeSet = this.getBaseElementTypeSet();
    ElementTypeSetConfiguration _baseUMLElementTypeSet = this.getBaseUMLElementTypeSet();
    return (!Objects.equal(_baseElementTypeSet, _baseUMLElementTypeSet));
  }
  
  public boolean isDiagramSpecific(final ElementTypeConfiguration type) {
    String _hint = type.getHint();
    return this.isVisualID(_hint);
  }
  
  public boolean hasSemanticSupertype(final ElementTypeConfiguration type) {
    boolean _and = false;
    boolean _isDiagramSpecific = this.isDiagramSpecific(type);
    if (!_isDiagramSpecific) {
      _and = false;
    } else {
      boolean _isSuppressSemanticSuperElementTypes = this._identifiers.isSuppressSemanticSuperElementTypes();
      boolean _not = (!_isSuppressSemanticSuperElementTypes);
      _and = _not;
    }
    return _and;
  }
  
  private boolean isVisualID(final String string) {
    boolean _and = false;
    boolean _isNullOrEmpty = StringExtensions.isNullOrEmpty(string);
    boolean _not = (!_isNullOrEmpty);
    if (!_not) {
      _and = false;
    } else {
      boolean _or = false;
      boolean _or_1 = false;
      boolean _or_2 = false;
      boolean _or_3 = false;
      boolean _or_4 = false;
      Matcher _matcher = UMLElementTypes.VISUAL_ID_PATTERN.matcher(string);
      boolean _matches = _matcher.matches();
      if (_matches) {
        _or_4 = true;
      } else {
        boolean _contains = string.contains("Diagram");
        _or_4 = _contains;
      }
      if (_or_4) {
        _or_3 = true;
      } else {
        boolean _contains_1 = string.contains("Shape");
        _or_3 = _contains_1;
      }
      if (_or_3) {
        _or_2 = true;
      } else {
        boolean _contains_2 = string.contains("Edge");
        _or_2 = _contains_2;
      }
      if (_or_2) {
        _or_1 = true;
      } else {
        boolean _contains_3 = string.contains("Label");
        _or_1 = _contains_3;
      }
      if (_or_1) {
        _or = true;
      } else {
        boolean _contains_4 = string.contains("Compartment");
        _or = _contains_4;
      }
      _and = _or;
    }
    return _and;
  }
  
  public Iterable<? extends ElementTypeConfiguration> getDiagramSpecificElementTypes(final org.eclipse.uml2.uml.Class metaclass) {
    Iterable<? extends ElementTypeConfiguration> _xifexpression = null;
    boolean _isDiagramSpecific = this.isDiagramSpecific();
    boolean _not = (!_isDiagramSpecific);
    if (_not) {
      ElementTypeSetConfiguration _baseUMLElementTypeSet = this.getBaseUMLElementTypeSet();
      EList<ElementTypeConfiguration> _elementTypeConfigurations = _baseUMLElementTypeSet.getElementTypeConfigurations();
      final Function1<ElementTypeConfiguration, Boolean> _function = new Function1<ElementTypeConfiguration, Boolean>() {
        public Boolean apply(final ElementTypeConfiguration it) {
          boolean _and = false;
          boolean _validType = UMLElementTypes.this.validType(it);
          if (!_validType) {
            _and = false;
          } else {
            String _identifier = it.getIdentifier();
            String _elementTypeID = UMLElementTypes.this.getElementTypeID(metaclass);
            boolean _equals = Objects.equal(_identifier, _elementTypeID);
            _and = _equals;
          }
          return Boolean.valueOf(_and);
        }
      };
      _xifexpression = IterableExtensions.<ElementTypeConfiguration>filter(_elementTypeConfigurations, _function);
    } else {
      ElementTypeSetConfiguration _baseElementTypeSet = this.getBaseElementTypeSet();
      EList<ElementTypeConfiguration> _elementTypeConfigurations_1 = _baseElementTypeSet.getElementTypeConfigurations();
      Iterable<SpecializationTypeConfiguration> _filter = Iterables.<SpecializationTypeConfiguration>filter(_elementTypeConfigurations_1, SpecializationTypeConfiguration.class);
      final Function1<SpecializationTypeConfiguration, Boolean> _function_1 = new Function1<SpecializationTypeConfiguration, Boolean>() {
        public Boolean apply(final SpecializationTypeConfiguration it) {
          boolean _and = false;
          boolean _validType = UMLElementTypes.this.validType(it);
          if (!_validType) {
            _and = false;
          } else {
            EList<String> _specializedTypesID = it.getSpecializedTypesID();
            String _elementTypeID = UMLElementTypes.this.getElementTypeID(metaclass);
            boolean _contains = _specializedTypesID.contains(_elementTypeID);
            _and = _contains;
          }
          return Boolean.valueOf(_and);
        }
      };
      _xifexpression = IterableExtensions.<SpecializationTypeConfiguration>filter(_filter, _function_1);
    }
    return _xifexpression;
  }
  
  private boolean canContain(final IElementType containerType, final EClass containedEClass) {
    EClass _eClass = containerType.getEClass();
    EList<EReference> _eAllContainments = _eClass.getEAllContainments();
    final Function1<EReference, Boolean> _function = new Function1<EReference, Boolean>() {
      public Boolean apply(final EReference it) {
        EClass _eReferenceType = it.getEReferenceType();
        return Boolean.valueOf(_eReferenceType.isSuperTypeOf(containedEClass));
      }
    };
    return IterableExtensions.<EReference>exists(_eAllContainments, _function);
  }
  
  private boolean canContain(final ElementTypeConfiguration containerType, final EClass containedEClass) {
    ElementTypeRegistry _instance = ElementTypeRegistry.getInstance();
    String _identifier = containerType.getIdentifier();
    IElementType _type = _instance.getType(_identifier);
    boolean _canContain = false;
    if (_type!=null) {
      _canContain=this.canContain(_type, containedEClass);
    }
    return _canContain;
  }
  
  protected boolean _canContainType(final ElementTypeConfiguration containerType, final ElementTypeConfiguration containedTypeConfiguration) {
    return false;
  }
  
  protected boolean _canContainType(final ElementTypeConfiguration containerType, final MetamodelTypeConfiguration containedTypeConfiguration) {
    EClass _eClass = containedTypeConfiguration.getEClass();
    return this.canContain(containerType, _eClass);
  }
  
  protected boolean _canContainType(final ElementTypeConfiguration containerType, final SpecializationTypeConfiguration containedTypeConfiguration) {
    EList<String> _specializedTypesID = containedTypeConfiguration.getSpecializedTypesID();
    final Function1<String, Boolean> _function = new Function1<String, Boolean>() {
      public Boolean apply(final String supertype) {
        ElementTypeRegistry _instance = ElementTypeRegistry.getInstance();
        IElementType _type = _instance.getType(supertype);
        EClass _eClass = _type.getEClass();
        return Boolean.valueOf(UMLElementTypes.this.canContain(containerType, _eClass));
      }
    };
    return IterableExtensions.<String>exists(_specializedTypesID, _function);
  }
  
  public boolean isRelationship(final IElementType elementType) {
    boolean _or = false;
    EClass _eClass = elementType.getEClass();
    boolean _equals = Objects.equal(_eClass, null);
    if (_equals) {
      _or = true;
    } else {
      EClass _eClass_1 = elementType.getEClass();
      boolean _isRelationship = this._uML.isRelationship(_eClass_1);
      _or = _isRelationship;
    }
    return _or;
  }
  
  public boolean isRelationship(final ElementTypeConfiguration elementType) {
    boolean _or = false;
    EClass _metaclass = this.getMetaclass(elementType);
    boolean _equals = Objects.equal(_metaclass, null);
    if (_equals) {
      _or = true;
    } else {
      EClass _metaclass_1 = this.getMetaclass(elementType);
      boolean _isRelationship = this._uML.isRelationship(_metaclass_1);
      _or = _isRelationship;
    }
    return _or;
  }
  
  private EClass _getMetaclass(final ElementTypeConfiguration elementType) {
    return null;
  }
  
  private EClass _getMetaclass(final SpecializationTypeConfiguration elementType) {
    EClass _xblockexpression = null;
    {
      ElementTypeRegistry _instance = ElementTypeRegistry.getInstance();
      String _identifier = elementType.getIdentifier();
      IElementType registered = _instance.getType(_identifier);
      EClass _xifexpression = null;
      boolean _notEquals = (!Objects.equal(registered, null));
      if (_notEquals) {
        _xifexpression = this.getMetaclass(registered);
      } else {
        EList<String> _specializedTypesID = elementType.getSpecializedTypesID();
        final Function1<String, IElementType> _function = new Function1<String, IElementType>() {
          public IElementType apply(final String it) {
            ElementTypeRegistry _instance = ElementTypeRegistry.getInstance();
            return _instance.getType(it);
          }
        };
        List<IElementType> _map = ListExtensions.<String, IElementType>map(_specializedTypesID, _function);
        Iterable<IElementType> _filterNull = IterableExtensions.<IElementType>filterNull(_map);
        IElementType _head = IterableExtensions.<IElementType>head(_filterNull);
        EClass _eClass = null;
        if (_head!=null) {
          _eClass=_head.getEClass();
        }
        _xifexpression = _eClass;
      }
      _xblockexpression = _xifexpression;
    }
    return _xblockexpression;
  }
  
  private EClass _getMetaclass(final MetamodelTypeConfiguration elementType) {
    return elementType.getEClass();
  }
  
  private EClass _getMetaclass(final IElementType elementType) {
    return null;
  }
  
  private EClass _getMetaclass(final IMetamodelType elementType) {
    return elementType.getEClass();
  }
  
  private EClass _getMetaclass(final ISpecializationType elementType) {
    IMetamodelType _metamodelType = elementType.getMetamodelType();
    EClass _eClass = null;
    if (_metamodelType!=null) {
      _eClass=_metamodelType.getEClass();
    }
    return _eClass;
  }
  
  private boolean canSourceTo(final IElementType sourceType, final EClass relationshipEClass) {
    Iterable<EReference> _sourceReferences = this._uML.getSourceReferences(relationshipEClass);
    final Function1<EReference, Boolean> _function = new Function1<EReference, Boolean>() {
      public Boolean apply(final EReference it) {
        EClass _eReferenceType = it.getEReferenceType();
        EClass _eClass = sourceType.getEClass();
        return Boolean.valueOf(_eReferenceType.isSuperTypeOf(_eClass));
      }
    };
    return IterableExtensions.<EReference>exists(_sourceReferences, _function);
  }
  
  private boolean canSourceTo(final ElementTypeConfiguration sourceType, final EClass relationshipEClass) {
    ElementTypeRegistry _instance = ElementTypeRegistry.getInstance();
    String _identifier = null;
    if (sourceType!=null) {
      _identifier=sourceType.getIdentifier();
    }
    IElementType _type = _instance.getType(_identifier);
    boolean _canSourceTo = false;
    if (_type!=null) {
      _canSourceTo=this.canSourceTo(_type, relationshipEClass);
    }
    return _canSourceTo;
  }
  
  protected boolean _canSourceToType(final ElementTypeConfiguration sourceType, final ElementTypeConfiguration relationshipTypeConfiguration) {
    return false;
  }
  
  protected boolean _canSourceToType(final ElementTypeConfiguration sourceType, final MetamodelTypeConfiguration relationshipTypeConfiguration) {
    EClass _eClass = relationshipTypeConfiguration.getEClass();
    return this.canSourceTo(sourceType, _eClass);
  }
  
  protected boolean _canSourceToType(final ElementTypeConfiguration sourceType, final SpecializationTypeConfiguration relationshipTypeConfiguration) {
    EList<String> _specializedTypesID = relationshipTypeConfiguration.getSpecializedTypesID();
    final Function1<String, Boolean> _function = new Function1<String, Boolean>() {
      public Boolean apply(final String supertypeID) {
        boolean _xblockexpression = false;
        {
          ElementTypeRegistry _instance = ElementTypeRegistry.getInstance();
          final IElementType supertype = _instance.getType(supertypeID);
          boolean _and = false;
          boolean _and_1 = false;
          boolean _notEquals = (!Objects.equal(supertype, null));
          if (!_notEquals) {
            _and_1 = false;
          } else {
            EClass _eClass = supertype.getEClass();
            boolean _isRelationship = UMLElementTypes.this._uML.isRelationship(_eClass);
            _and_1 = _isRelationship;
          }
          if (!_and_1) {
            _and = false;
          } else {
            EClass _eClass_1 = supertype.getEClass();
            boolean _canSourceTo = UMLElementTypes.this.canSourceTo(sourceType, _eClass_1);
            _and = _canSourceTo;
          }
          _xblockexpression = _and;
        }
        return Boolean.valueOf(_xblockexpression);
      }
    };
    return IterableExtensions.<String>exists(_specializedTypesID, _function);
  }
  
  private boolean canTargetFrom(final IElementType targetType, final EClass relationshipEClass) {
    Iterable<EReference> _targetReferences = this._uML.getTargetReferences(relationshipEClass);
    final Function1<EReference, Boolean> _function = new Function1<EReference, Boolean>() {
      public Boolean apply(final EReference it) {
        EClass _eReferenceType = it.getEReferenceType();
        EClass _eClass = targetType.getEClass();
        return Boolean.valueOf(_eReferenceType.isSuperTypeOf(_eClass));
      }
    };
    return IterableExtensions.<EReference>exists(_targetReferences, _function);
  }
  
  private boolean canTargetFrom(final ElementTypeConfiguration targetType, final EClass relationshipEClass) {
    ElementTypeRegistry _instance = ElementTypeRegistry.getInstance();
    String _identifier = null;
    if (targetType!=null) {
      _identifier=targetType.getIdentifier();
    }
    IElementType _type = _instance.getType(_identifier);
    boolean _canTargetFrom = false;
    if (_type!=null) {
      _canTargetFrom=this.canTargetFrom(_type, relationshipEClass);
    }
    return _canTargetFrom;
  }
  
  protected boolean _canTargetFromType(final ElementTypeConfiguration targetType, final ElementTypeConfiguration relationshipTypeConfiguration) {
    return false;
  }
  
  protected boolean _canTargetFromType(final ElementTypeConfiguration targetType, final MetamodelTypeConfiguration relationshipTypeConfiguration) {
    EClass _eClass = relationshipTypeConfiguration.getEClass();
    return this.canTargetFrom(targetType, _eClass);
  }
  
  protected boolean _canTargetFromType(final ElementTypeConfiguration targetType, final SpecializationTypeConfiguration relationshipTypeConfiguration) {
    EList<String> _specializedTypesID = relationshipTypeConfiguration.getSpecializedTypesID();
    final Function1<String, Boolean> _function = new Function1<String, Boolean>() {
      public Boolean apply(final String supertypeID) {
        boolean _xblockexpression = false;
        {
          ElementTypeRegistry _instance = ElementTypeRegistry.getInstance();
          final IElementType supertype = _instance.getType(supertypeID);
          boolean _and = false;
          boolean _and_1 = false;
          boolean _notEquals = (!Objects.equal(supertype, null));
          if (!_notEquals) {
            _and_1 = false;
          } else {
            EClass _eClass = supertype.getEClass();
            boolean _isRelationship = UMLElementTypes.this._uML.isRelationship(_eClass);
            _and_1 = _isRelationship;
          }
          if (!_and_1) {
            _and = false;
          } else {
            EClass _eClass_1 = supertype.getEClass();
            boolean _canTargetFrom = UMLElementTypes.this.canTargetFrom(targetType, _eClass_1);
            _and = _canTargetFrom;
          }
          _xblockexpression = _and;
        }
        return Boolean.valueOf(_xblockexpression);
      }
    };
    return IterableExtensions.<String>exists(_specializedTypesID, _function);
  }
  
  public boolean canContainType(final ElementTypeConfiguration containerType, final ElementTypeConfiguration containedTypeConfiguration) {
    if (containedTypeConfiguration instanceof MetamodelTypeConfiguration) {
      return _canContainType(containerType, (MetamodelTypeConfiguration)containedTypeConfiguration);
    } else if (containedTypeConfiguration instanceof SpecializationTypeConfiguration) {
      return _canContainType(containerType, (SpecializationTypeConfiguration)containedTypeConfiguration);
    } else if (containedTypeConfiguration != null) {
      return _canContainType(containerType, containedTypeConfiguration);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(containerType, containedTypeConfiguration).toString());
    }
  }
  
  private EClass getMetaclass(final Object elementType) {
    if (elementType instanceof MetamodelTypeConfiguration) {
      return _getMetaclass((MetamodelTypeConfiguration)elementType);
    } else if (elementType instanceof SpecializationTypeConfiguration) {
      return _getMetaclass((SpecializationTypeConfiguration)elementType);
    } else if (elementType instanceof ElementTypeConfiguration) {
      return _getMetaclass((ElementTypeConfiguration)elementType);
    } else if (elementType instanceof IMetamodelType) {
      return _getMetaclass((IMetamodelType)elementType);
    } else if (elementType instanceof ISpecializationType) {
      return _getMetaclass((ISpecializationType)elementType);
    } else if (elementType instanceof IElementType) {
      return _getMetaclass((IElementType)elementType);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(elementType).toString());
    }
  }
  
  public boolean canSourceToType(final ElementTypeConfiguration sourceType, final ElementTypeConfiguration relationshipTypeConfiguration) {
    if (relationshipTypeConfiguration instanceof MetamodelTypeConfiguration) {
      return _canSourceToType(sourceType, (MetamodelTypeConfiguration)relationshipTypeConfiguration);
    } else if (relationshipTypeConfiguration instanceof SpecializationTypeConfiguration) {
      return _canSourceToType(sourceType, (SpecializationTypeConfiguration)relationshipTypeConfiguration);
    } else if (relationshipTypeConfiguration != null) {
      return _canSourceToType(sourceType, relationshipTypeConfiguration);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(sourceType, relationshipTypeConfiguration).toString());
    }
  }
  
  public boolean canTargetFromType(final ElementTypeConfiguration targetType, final ElementTypeConfiguration relationshipTypeConfiguration) {
    if (relationshipTypeConfiguration instanceof MetamodelTypeConfiguration) {
      return _canTargetFromType(targetType, (MetamodelTypeConfiguration)relationshipTypeConfiguration);
    } else if (relationshipTypeConfiguration instanceof SpecializationTypeConfiguration) {
      return _canTargetFromType(targetType, (SpecializationTypeConfiguration)relationshipTypeConfiguration);
    } else if (relationshipTypeConfiguration != null) {
      return _canTargetFromType(targetType, relationshipTypeConfiguration);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(targetType, relationshipTypeConfiguration).toString());
    }
  }
}
